/***********************************************************************

	$Id: cbuf.c,v 1.4 2009/09/02 10:31:08 warme Exp $

	File:	cbuf.c
	Rev:	a-1
	Date:	12/17/2005

	Copyright (c) 2005, 2009 by David M. Warme.  All rights
	reserved.

************************************************************************

	A simple "rubber-band" buffer for character strings.

************************************************************************

	Modification Log:

	a-1:	12/17/2005	warme
		: Created.

************************************************************************/

#include "cbuf.h"
#include "logic.h"
#include "memory.h"

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>


/*
 * Initialize an empty CBuf.
 */

	void
init_cbuf (

struct CBuf *	bufp
)
{
	bufp -> count	= 0;
	bufp -> next	= NULL;
	bufp -> last	= NULL;
}


/*
 * Free up a CBuf chain.
 */

	void
free_cbuf (

struct CBuf *	bufp
)
{
struct CBuf *	tmp;

	while (bufp NE NULL) {
		tmp = bufp -> next;
		free (bufp);
		bufp = tmp;
	}
}

/*
 * Append the given character to the given CBuf.
 */

	void
put_cbuf (

struct CBuf *	bufp,
char		c
)
{
struct CBuf *	last;
struct CBuf *	newp;

	last = bufp -> last;
	if (last EQ NULL) {
		last = bufp;
	}

	if (last -> count >= MAX_CBUF) {
		/* Need to append another buffer onto the end. */
		newp = NEW (struct CBuf);
		newp -> count	= 0;
		newp -> next	= NULL;
		newp -> last	= NULL;

		if (last EQ bufp) {
			bufp -> next = newp;
			bufp -> last = newp;
		}
		else {
			last -> next = newp;
			bufp -> last = newp;
		}
		last = newp;
	}
	last -> buf [last -> count++] = c;
}
