/***********************************************************************

	$Id: config.y,v 1.4 2009/09/02 10:42:41 warme Exp $

	File:	config.y
	Rev:	a-2
	Date:	09/02/2009

	Copyright (c) 2006, 2009 by David M. Warme.  All rights
	reserved.

************************************************************************

	Bison grammer for eeg's configuration file.

************************************************************************

	Modification Log:

	a-1:	03/15/2006	warme
		: Created.
	a-2:	09/02/2009	warme
		: Added "linear" channel definition option.

************************************************************************/


%{
#include "logic.h"
#include "memory.h"
#include "pnodes.h"
#include <stddef.h>
#include <stdio.h>

struct ConfigFilePnode *	root_pnode;

#define	DEF_REV(func, tag) \
 static struct tag * func (struct tag * p) \
 { \
 struct tag *	revlist; \
 struct tag *	tmp; \
	revlist = NULL; \
	while (p NE NULL) { \
		tmp = p -> next; \
		p -> next = revlist; \
		revlist = p; \
		p = tmp; \
	} \
	return (revlist); \
 }

DEF_REV (reverse_chan_def_options,		ChanDefOptionPnode);
DEF_REV (reverse_field_defs,			FieldDefPnode);
DEF_REV (reverse_logical_field_names,		LogicalFieldNamePnode);
DEF_REV (reverse_plot_options,			PlotOptionPnode);
DEF_REV (reverse_source_defs,			SourceDefPnode);
DEF_REV (reverse_source_def_items,		SourceDefItemPnode);

#undef DEF_REV

	static
	void
yyerror (

const char *	msg
)
{
	fprintf (stderr, "%s\n", msg);
}

%}

%union {
	struct ChanDefPnode *		y_chandef;
	struct ChanDefOptionPnode *	y_chandefopt;
	double				y_double;
	struct FieldDefPnode *		y_flddef;
	struct LogicalFieldNamePnode *	y_logfldname;
	long				y_long;
	struct Name *			y_name;
	struct PlotDefPnode *		y_plotdef;
	struct PlotOptionPnode *	y_plotoption;
	struct SourceDefPnode *		y_srcdef;
	struct SourceDefItemPnode *	y_srcdefitem;
	struct VersionDefPnode *	y_versdef;
};

/* Tokens with no type. */

%token			BOTTOM_TITLE
%token			CHAN
%token			COLON
%token			COMMA
%token			DATA
%token			ID
%token			LANDSCAPE
%token			LBRACE
%token			LBRACK
%token			LINEAR
%token			LPAREN
%token			MAGNIFY
%token			MINUS
%token			ORIENTATION
%token			PLOT
%token			PLUS
%token			PORTRAIT
%token			RBRACE
%token			RBRACK
%token			RPAREN
%token			SCATTER
%token			TOP_TITLE
%token			VERSION

/* Tokens with typed data. */

%token	<y_double>	FCONST
%token	<y_long>	ICONST
%token	<y_name>	IDENT
%token	<y_name>	STRING

/* Non-terminal types. */

%type	<y_name>	bottom_title_option
%type	<y_chandef>	chan_def
%type	<y_name>	chan_def_data_option
%type	<y_chandefopt>	chan_def_linear_option
%type	<y_double>	chan_def_magnify_option
%type	<y_chandefopt>	chan_def_option
%type	<y_chandefopt>	chan_def_option_list
%type	<y_flddef>	field_def
%type	<y_flddef>	field_def_list
%type	<y_double>	fnum
%type	<y_long>	orientation_option
%type	<y_plotdef>	plot_def
%type	<y_plotoption>	plot_option
%type	<y_plotoption>	plot_option_list
%type	<y_srcdef>	source_def
%type	<y_srcdefitem>	source_def_item
%type	<y_srcdefitem>	source_def_item_list
%type	<y_srcdef>	source_def_list
%type	<y_logfldname>	string_list
%type	<y_name>	top_title_option
%type	<y_versdef>	version_def

%start	config_file

%%

config_file: source_def_list
	  { root_pnode = NEW (struct ConfigFilePnode);
	    root_pnode -> srcdefs = reverse_source_defs ($1);
	  }
	;

source_def_list:
	  /* empty */
	  {
		$$ = NULL;
	  }
	| source_def_list source_def
	  {
		$$ = $2;
		$$ -> next = $1;
	  }
	| source_def_list error source_def
	  {
		$$ = $3;
		$$ -> next = $1;
	  }
	;

source_def:
	  ID COLON STRING LBRACE source_def_item_list RBRACE
	  { $$ = NEW (struct SourceDefPnode);
	    $$ -> src_name	= $3;
	    $$ -> items		= reverse_source_def_items ($5);
	    $$ -> next		= NULL;
	  }
	;

source_def_item_list:
	  /* empty */
	  { $$ = NULL; }
	| source_def_item_list source_def_item
	  { $$ = $2;
	    $$ -> next = $1;
	  }
	| source_def_item_list error source_def_item
	  { $$ = $3;
	    $$ -> next = $1;
	  }
	;

source_def_item:
	  version_def
	  { $$ = NEW (struct SourceDefItemPnode);
	    $$ -> type		= SRCDEF_VERSION;
	    $$ -> u.version_def	= $1;
	    $$ -> next		= NULL;
	  }
	| plot_def
	  { $$ = NEW (struct SourceDefItemPnode);
	    $$ -> type		= SRCDEF_PLOT;
	    $$ -> u.plot_def	= $1;
	    $$ -> next		= NULL;
	  }
	;

version_def:
	  VERSION COLON STRING LBRACE field_def_list RBRACE
	  { $$ = NEW (struct VersionDefPnode);
	    $$ -> version	= $3;
	    $$ -> field_defs	= reverse_field_defs ($5);
	  }
	;

field_def_list:
	  /* empty */
	  { $$ = NULL; }
	| field_def_list field_def
	  { $$ = $2;
	    $$ -> next = $1;
	  }
	| field_def_list error field_def
	  { $$ = $3;
	    $$ -> next = $1;
	  }
	;

field_def:
	  STRING
	  { struct LogicalFieldNamePnode *	p;
	    p = NEW (struct LogicalFieldNamePnode);
	    p -> name	= $1;
	    p -> next	= NULL;
	    $$ = NEW (struct FieldDefPnode);
	    $$ -> logical_names	= p;
	    $$ -> next		= NULL;
	  }
	| LPAREN string_list RPAREN
	  { $$ = NEW (struct FieldDefPnode);
	    $$ -> logical_names	= reverse_logical_field_names ($2);
	    $$ -> next		= NULL;
	  }
	;

string_list:
	  /* empty */
	  { $$ = NULL; }
	| string_list STRING
	  { $$ = NEW (struct LogicalFieldNamePnode);
	    $$ -> name = $2;
	    $$ -> next = $1;
	  }
	| string_list error STRING
	  { $$ = NEW (struct LogicalFieldNamePnode);
	    $$ -> name = $3;
	    $$ -> next = $1;
	  }
	;

plot_def:
	  PLOT COLON STRING
	  { $$ = NEW (struct PlotDefPnode);
	    $$ -> name		= $3;
	    $$ -> options	= NULL;
	  }
	| PLOT COLON STRING LBRACE plot_option_list RBRACE
	  { $$ = NEW (struct PlotDefPnode);
	    $$ -> name		= $3;
	    $$ -> options	= reverse_plot_options ($5);
	  }
	| PLOT COLON STRING LBRACE plot_option_list error RBRACE
	  { $$ = NEW (struct PlotDefPnode);
	    $$ -> name		= $3;
	    $$ -> options	= reverse_plot_options ($5);
	  }
	;

plot_option_list:
	  /* empty */
	  { $$ = NULL; }
	| plot_option_list plot_option
	  { $$ = $2;
	    $$ -> next = $1;
	  }
	| plot_option_list error plot_option
	  { $$ = $3;
	    $$ -> next = $1;
	  }
	;

plot_option:
	  chan_def
	  { $$ = NEW (struct PlotOptionPnode);
	    $$ -> type		= PLOTOPT_CHAN_DEF;
	    $$ -> u.chan_def	= $1;
	    $$ -> next		= NULL;
	  }
	| bottom_title_option
	  { $$ = NEW (struct PlotOptionPnode);
	    $$ -> type		= PLOTOPT_BOTTOM_TITLE;
	    $$ -> u.title	= $1;
	    $$ -> next		= NULL;
	  }
	| top_title_option
	  { $$ = NEW (struct PlotOptionPnode);
	    $$ -> type		= PLOTOPT_TOP_TITLE;
	    $$ -> u.title	= $1;
	    $$ -> next		= NULL;
	  }
	| orientation_option
	  { $$ = NEW (struct PlotOptionPnode);
	    $$ -> type		= PLOTOPT_ORIENTATION;
	    $$ -> u.landscape	= ($1 NE 0);
	    $$ -> next		= NULL;
	  }
	;

chan_def:
	  CHAN COLON STRING
	  { $$ = NEW (struct ChanDefPnode);
	    $$ -> name		= $3;
	    $$ -> options	= NULL;
	    $$ -> next		= NULL;
	  }
	| CHAN COLON STRING LBRACE chan_def_option_list RBRACE
	  { $$ = NEW (struct ChanDefPnode);
	    $$ -> name		= $3;
	    $$ -> options	= reverse_chan_def_options ($5);
	    $$ -> next		= NULL;
	  }
	| CHAN COLON STRING LBRACE chan_def_option_list error RBRACE
	  { $$ = NEW (struct ChanDefPnode);
	    $$ -> name		= $3;
	    $$ -> options	= reverse_chan_def_options ($5);
	    $$ -> next		= NULL;
	  }
	;

chan_def_option_list:
	  /* empty */
	  { $$ = NULL; }
	| chan_def_option_list chan_def_option
	  { $$ = $2;
	    $$ -> next = $1;
	  }
	| chan_def_option_list error chan_def_option
	  { $$ = $3;
	    $$ -> next = $1;
	  }
	;

chan_def_option:
	  chan_def_data_option
	  { $$ = NEW (struct ChanDefOptionPnode);
	    $$ -> type		= CHANOPT_DATA;
	    $$ -> u.data_name	= $1;
	    $$ -> next		= NULL;
	  }
	| chan_def_linear_option
	  { $$ = $1; }
	| chan_def_magnify_option
	  { $$ = NEW (struct ChanDefOptionPnode);
	    $$ -> type			= CHANOPT_MAGNIFY;
	    $$ -> u.magnification	= $1;
	    $$ -> next			= NULL;
	  }
	| SCATTER
	  { $$ = NEW (struct ChanDefOptionPnode);
	    $$ -> type = CHANOPT_SCATTER;
	    $$ -> u.magnification	= 0.0;
	    $$ -> next			= NULL;
	  }
	;

chan_def_data_option:
	  DATA COLON STRING
	  { $$ = $3; }
	;

chan_def_linear_option:
	  LINEAR COLON LBRACK fnum COMMA fnum RBRACK
	  { $$ = NEW (struct ChanDefOptionPnode);
	    $$ -> type			= CHANOPT_LINEAR;
	    $$ -> u.linear.min		= $4;
	    $$ -> u.linear.max		= $6;
	    $$ -> next			= NULL;
	  }
	;

chan_def_magnify_option:
	  MAGNIFY COLON fnum
	  { $$ = $3; }
	;

bottom_title_option:
	  BOTTOM_TITLE COLON STRING
	  { $$ = $3; }
	;

top_title_option:
	  TOP_TITLE COLON STRING
	  { $$ = $3; }
	;

orientation_option:
	  ORIENTATION COLON PORTRAIT
	  { $$ = FALSE; }
	| ORIENTATION COLON LANDSCAPE
	  { $$ = TRUE; }
	;

fnum:
	  ICONST
	  { $$ = (double) $1; }
	| PLUS ICONST
	  { $$ = (double) $2; }
	| MINUS ICONST
	  { $$ = - (double) $2; }
	| FCONST
	  { $$ = $1; }
	| PLUS FCONST
	  { $$ = $2; }
	| MINUS FCONST
	  { $$ = - $2; }
	;
