/***********************************************************************

	$Id: datum.h,v 1.4 2009/09/02 10:31:08 warme Exp $

	File:	datum.h
	Rev:	a-1
	Date:	12/17/2005

	Copyright (c) 2005, 2009 by David M. Warme.  All rights
	reserved.

************************************************************************

	Data structures to represent numeric input values.

************************************************************************

	Modification Log:

	a-1:	12/17/2005	warme
		: Created.

************************************************************************/

#ifndef DATUM_H
#define DATUM_H

#include <stdio.h>


/* Structure to hold a single data value. */

struct Datum {
	int		type;
	double		value;
};

#define	DATUM_TYPE_VALUE	0	/* A valid numeric value */
#define	DATUM_TYPE_PINF		1	/* Plus infinity */
#define	DATUM_TYPE_MINF		2	/* Minus infinity */
#define	DATUM_TYPE_NAN		3	/* Not-a-Number */

/* A "rubber band" datum buffer. */

#define MAX_DBUF	256

struct DBuf {
	int		count;		/* Number of items in this buffer */
	struct DBuf *	next;		/* Next buffer in chain */
	struct DBuf *	last;		/* Last buffer in chain */
	struct Datum	buf [MAX_DBUF];	/* Buffer of data items */
};


/* Function prototypes. */

extern void		append_datum (struct DBuf **	hookp,
				      struct Datum *	datum);
extern void		free_dbuf (struct DBuf * bufp);
extern void		linearize_dbuf (struct DBuf *	bufp,
					struct Datum **	array,
					int *		count);
extern int		read_datum (FILE * fp, struct Datum * datum);

#endif
