/***********************************************************************

	$Id: paper.c,v 1.4 2009/09/02 10:44:10 warme Exp $

	File:	paper.c
	Rev:	a-2
	Date:	09/02/2009

	Copyright (c) 2005, 2009 by David M. Warme.  All rights
	reserved.

************************************************************************

	Encapsulate different paper sizes.

************************************************************************

	Modification Log:

	a-1:	12/17/2005	warme
		: Created.
	a-2:	09/02/2009	warme
		: Get default paper size from configure script.

************************************************************************/

#include "paper.h"
#include "config.h"
#include "logic.h"
#include "mystring.h"

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>

extern char *		me;

/*
 * Return the default paper size.
 */

	const struct PaperSize *
get_default_paper_size (void)

{
const char *			default_paper;
const struct PaperSize *	ps;

	default_paper = DEFAULT_PAPER_SIZE_STRING;

	ps = get_paper_size (default_paper);

	return (ps);
}

/*
 * A table of various paper sizes.
 */

static const struct PaperSize		paper_sizes [] = {
 /* Letter = 8.5" by 11.0". */
 {	612.0,		792.0,		"Letter"	},

 /* Legal = 8.5" by 14.0". */
 {	612.0,		1008.0,		"Legal"		},

 /* Ledger = 11.0" by 17.0". */
 {	792.0,		1224.0,		"Ledger"	},

 /* A0 = 840.896 mm by 1189.207 mm. */
 {	2383.64338,	3370.980798,	"A0"		},

 /* A1 = 594.603 mm by 840.896 mm. */
 {	1685.490399,	2383.64338,	"A1"		},

 /* A2 = 420.448 mm by 594.603 mm. */
 {	1191.821691,	1685.490399,	"A2"		},

 /* A3 = 297.302 mm by 420.448 mm. */
 {	842.7451996,	1191.821691,	"A3"		},

 /* A4 = 210.2241 mm by 297.302 mm. */
 {	595.910845455,	842.7451996,	"A4"		},
};

/* A static buffer for holding a custom paper size. */
static struct PaperSize		custom_paper_size;

/*
 * Return the paper size object for the given paper size name.
 */

	const struct PaperSize *
get_paper_size (

const char *	name
)
{
int			i, n;
int			valid;
const struct PaperSize * ps;
struct PaperSize *	newp;
char *			p2;
char *			endp;
double			x, y;

	n = sizeof (paper_sizes) / sizeof (paper_sizes [0]);

	for (i = 0; i < n; i++) {
		ps = &paper_sizes [i];
		if (mystrcasecmp (name, ps -> name) EQ 0) {
			return (ps);
		}
	}

	/* The given name is not one of our known paper size names.	*/
	/* Perhaps it is of the form "X,Y" where X and Y are in		*/
	/* Postscript units.						*/
	x = 0.0;
	y = 0.0;

	valid = 0;
	x = strtod (name, &endp);
	if ((endp > name) AND (*endp EQ ',')) {
		p2 = endp + 1;
		y = strtod (p2, &endp);
		if ((endp > p2) AND (*endp EQ '\0')) {
			valid = 1;
		}
	}
	if (NOT valid) {
		fprintf (stderr, "%s: Paper size `%s' is unrecognized.\n",
			 me, name);
		exit (1);
	}

	/* Make up a custom paper size. */

	newp = &custom_paper_size;

	newp -> x	= x;
	newp -> y 	= y;
	newp -> name	= mystrdup (name);

	return (newp);
}
