/***********************************************************************

	$Id: pnodes.h,v 1.4 2009/09/02 10:42:42 warme Exp $

	File:	pnodes.h
	Rev:	a-2
	Date:	09/02/2009

	Copyright (c) 2006, 2009 by David M. Warme.  All rights
	reserved.

************************************************************************

	Parse tree for the eeg configuration file.

************************************************************************

	Modification Log:

	a-1:	05/29/2006	warme
		: Created.
	a-2:	09/02/2009	warme
		: Added "linear" channel definition option.

************************************************************************/

#ifndef PNODES_H
#define PNODES_H


/* Top-level parse tree node for the whole config file. */

extern struct ConfigFilePnode *		root_pnode;
extern int				yyparse ();

struct ConfigFilePnode {
	struct SourceDefPnode *	srcdefs;
};

struct SourceDefPnode {
	struct Name *			src_name;
	struct SourceDefItemPnode *	items;
	struct SourceDefPnode *		next;
};

enum SrcDefItemType {
	SRCDEF_VERSION,
	SRCDEF_PLOT,
};

struct SourceDefItemPnode {
	enum SrcDefItemType		type;
	union {
		struct VersionDefPnode * version_def;
		struct PlotDefPnode *	 plot_def;
	} u;
	struct SourceDefItemPnode *	next;
};

struct VersionDefPnode {
	struct Name *		version;
	struct FieldDefPnode *	field_defs;
};

struct FieldDefPnode {
	struct LogicalFieldNamePnode *	logical_names;
	struct FieldDefPnode *		next;
};

struct LogicalFieldNamePnode {
	struct Name *			name;
	struct LogicalFieldNamePnode *	next;
};

struct PlotDefPnode {
	struct Name *			name;
	struct PlotOptionPnode *	options;
};

enum PlotOptionType {
	PLOTOPT_CHAN_DEF,
	PLOTOPT_BOTTOM_TITLE,
	PLOTOPT_TOP_TITLE,
	PLOTOPT_ORIENTATION,
};

struct PlotOptionPnode {
	enum PlotOptionType		type;
	union {
		struct ChanDefPnode *	chan_def;
		struct Name *		title;
		int			landscape;
	} u;
	struct PlotOptionPnode *	next;
};

struct ChanDefPnode {
	struct Name *			name;
	struct ChanDefOptionPnode *	options;
	struct ChanDefPnode *		next;
};

enum ChanDefOptionType {
	CHANOPT_DATA,
	CHANOPT_LINEAR,
	CHANOPT_MAGNIFY,
	CHANOPT_SCATTER,
};

struct ChanDefOptionPnode {
	enum ChanDefOptionType		type;
	union {
		struct Name *		data_name;
		double			magnification;
		struct {
			double		min;
			double		max;
		} linear;
	} u;
	struct ChanDefOptionPnode *	next;
};

#endif
